@@ -17,14 +17,14 @@
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import utils
 from neutron.conf.agent import common
-# from neutron.plugins.openvswitch.common import constants as ovs_consts
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
-    as ovs_consts
+from neutron_lib.plugins.ml2 import ovs_constants as n_ovs_consts
+
 from neutron_taas.services.taas.agents.extensions import taas as taas_base
 import neutron_taas.services.taas.drivers.linux.ovs_constants \
     as taas_ovs_consts
 import neutron_taas.services.taas.drivers.linux.ovs_utils as taas_ovs_utils
 from oslo_config import cfg
+from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 
 
@@ -35,12 +35,12 @@ TaaS_DRIVER_NAME = 'Taas OVS driver'
 
 class OVSBridge_tap_extension(ovs_lib.OVSBridge):
     def __init__(self, br_name, root_helper):
-        super(OVSBridge_tap_extension, self).__init__(br_name)
+        super().__init__(br_name)
 
 
 class OvsTaasDriver(taas_base.TaasAgentDriver):
     def __init__(self):
-        super(OvsTaasDriver, self).__init__()
+        super().__init__()
         LOG.debug("Initializing Taas OVS Driver")
         self.agent_api = None
         self.root_helper = common.get_root_helper(cfg.CONF)
@@ -196,14 +196,21 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                                  "output:NXM_OF_IN_PORT[])" %
                                  taas_ovs_consts.TAAS_SEND_UCAST))
 
-        return
-
     def consume_api(self, agent_api):
         self.agent_api = agent_api
 
-    def create_tap_service(self, tap_service):
-        taas_id = tap_service['taas_id']
-        port = tap_service['port']
+    @log_helpers.log_method_call
+    def create_tap_service(self, tap_service_msg):
+        """Create a tap service
+
+        :param tap_service_msg: a dict of the tap_service,
+                                taas_id which is the VLAN Id reserved for
+                                mirroring for this tap-service and the neutron
+                                port of the tap-service:
+                                {tap_service: {}, taas_id: VID, port: {}}
+        """
+        taas_id = tap_service_msg['taas_id']
+        port = tap_service_msg['port']
 
         # Get OVS port id for tap service port
         ovs_port = self.int_br.get_vif_port_by_id(port['id'])
@@ -237,8 +244,8 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                              actions="output:%s" % str(patch_tap_int_id))
 
         # Add flow(s) in br-tun
-        for tunnel_type in ovs_consts.TUNNEL_NETWORK_TYPES:
-            self.tun_br.add_flow(table=ovs_consts.TUN_TABLE[tunnel_type],
+        for tunnel_type in n_ovs_consts.TUNNEL_NETWORK_TYPES:
+            self.tun_br.add_flow(table=n_ovs_consts.TUN_TABLE[tunnel_type],
                                  priority=1,
                                  tun_id=taas_id,
                                  actions=(
@@ -256,21 +263,35 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
 
         #
         # Disable mac-address learning in the Linux bridge to which
-        # the OVS port is attached (via the veth pair). This will
-        # effectively turn the bridge into a hub, ensuring that all
-        # incoming mirrored traffic reaches the tap interface (used
-        # for attaching a VM to the bridge) irrespective of the
+        # the OVS port is attached (via the veth pair) if the system
+        # uses OVSHybridIptablesFirewallDriver (Linux bridge & OVS).
+        # This will effectively turn the bridge into a hub, ensuring
+        # that all incoming mirrored traffic reaches the tap interface
+        # (used for attaching a VM to the bridge) irrespective of the
         # destination mac addresses in mirrored packets.
         #
-        ovs_port_name = ovs_port.port_name
-        linux_br_name = ovs_port_name.replace('qvo', 'qbr')
-        utils.execute(['brctl', 'setageing', linux_br_name, 0],
-                      run_as_root=True)
 
-        return
+        # Get hybrid plug info
+        vif_details = port.get('binding:vif_details')
+        is_hybrid_plug = vif_details.get('ovs_hybrid_plug')
 
-    def delete_tap_service(self, tap_service):
-        taas_id = tap_service['taas_id']
+        if is_hybrid_plug:
+            ovs_port_name = ovs_port.port_name
+            linux_br_name = ovs_port_name.replace('qvo', 'qbr')
+            utils.execute(['brctl', 'setageing', linux_br_name, 0],
+                          run_as_root=True, privsep_exec=True)
+
+    @log_helpers.log_method_call
+    def delete_tap_service(self, tap_service_msg):
+        """Delete a tap service
+
+        :param tap_service_msg: a dict of the tap_service,
+                                taas_id which is the VLAN Id reserved for
+                                mirroring for this tap-service and the neutron
+                                port of the tap-service:
+                                {tap_service: {}, taas_id: VID, port: {}}
+        """
+        taas_id = tap_service_msg['taas_id']
 
         # Get patch port ID
         patch_int_tap_id = self.int_br.get_port_ofport('patch-int-tap')
@@ -288,8 +309,8 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                                  dl_vlan=taas_id)
 
         # Delete flow(s) from br-tun
-        for tunnel_type in ovs_consts.TUNNEL_NETWORK_TYPES:
-            self.tun_br.delete_flows(table=ovs_consts.TUN_TABLE[tunnel_type],
+        for tunnel_type in n_ovs_consts.TUNNEL_NETWORK_TYPES:
+            self.tun_br.delete_flows(table=n_ovs_consts.TUN_TABLE[tunnel_type],
                                      tun_id=taas_id)
 
         self.tun_br.delete_flows(table=taas_ovs_consts.TAAS_DST_CHECK,
@@ -298,12 +319,22 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
         self.tun_br.delete_flows(table=taas_ovs_consts.TAAS_SRC_CHECK,
                                  tun_id=taas_id)
 
-        return
+    @log_helpers.log_method_call
+    def create_tap_flow(self, tap_flow_msg):
+        """Create a tap flow
 
-    def create_tap_flow(self, tap_flow):
-        taas_id = tap_flow['taas_id']
-        port = tap_flow['port']
-        direction = tap_flow['tap_flow']['direction']
+        :param tap_flow_msg: a dict of the tap_flow, the mac of the port of
+                             the tap-flow, taas_id which is the VLAN Id
+                             reserved for mirroring for the tap-service
+                             associated with this tap-flow, the neutron port
+                             of the tap-flow, and the port of the
+                             tap-service:
+                             {tap_flow: {}, port_mac: '', taas_id: VID,
+                             port: {}, tap_service_port: {}}
+        """
+        taas_id = tap_flow_msg['taas_id']
+        port = tap_flow_msg['port']
+        direction = tap_flow_msg['tap_flow']['direction']
 
         # Get OVS port id for tap flow port
         ovs_port = self.int_br.get_vif_port_by_id(port['id'])
@@ -313,15 +344,15 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
         patch_int_tap_id = self.int_br.get_port_ofport('patch-int-tap')
 
         # Add flow(s) in br-int
-        if direction == 'OUT' or direction == 'BOTH':
+        if direction in ('OUT', 'BOTH'):
             self.int_br.add_flow(table=0,
                                  priority=20,
                                  in_port=ovs_port_id,
                                  actions="normal,mod_vlan_vid:%s,output:%s" %
                                  (str(taas_id), str(patch_int_tap_id)))
 
-        if direction == 'IN' or direction == 'BOTH':
-            port_mac = tap_flow['port_mac']
+        if direction in ('IN', 'BOTH'):
+            port_mac = tap_flow_msg['port_mac']
 
             #
             # Note: The ingress side flow (for unicast traffic) should
@@ -357,8 +388,8 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
             #                                    patch_int_tap_id)
 
         # Add flow(s) in br-tun
-        for tunnel_type in ovs_consts.TUNNEL_NETWORK_TYPES:
-            self.tun_br.add_flow(table=ovs_consts.TUN_TABLE[tunnel_type],
+        for tunnel_type in n_ovs_consts.TUNNEL_NETWORK_TYPES:
+            self.tun_br.add_flow(table=n_ovs_consts.TUN_TABLE[tunnel_type],
                                  priority=1,
                                  tun_id=taas_id,
                                  actions=(
@@ -374,23 +405,35 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                              actions="resubmit(,%s)" %
                              taas_ovs_consts.TAAS_SRC_RESPOND)
 
-        return
+    @log_helpers.log_method_call
+    def delete_tap_flow(self, tap_flow_msg):
+        """Delete a tap flow
 
-    def delete_tap_flow(self, tap_flow):
-        port = tap_flow['port']
-        direction = tap_flow['tap_flow']['direction']
+        :param tap_flow_msg: a dict of the tap_flow, the mac of the port of
+                             the tap-flow, taas_id which is the VLAN Id
+                             reserved for mirroring for the tap-service
+                             associated with this tap-flow, the neutron port
+                             of the tap-flow, the port of the
+                             tap-service, the list of source VLAN IDs, and
+                             VLAN filter list.
+                             {tap_flow: {}, port_mac: '', taas_id: VID,
+                             port: {}, tap_service_port: {},
+                             source_vlans_list: [], vlan_filter_list: []}
+        """
+        port = tap_flow_msg['port']
+        direction = tap_flow_msg['tap_flow']['direction']
 
         # Get OVS port id for tap flow port
         ovs_port = self.int_br.get_vif_port_by_id(port['id'])
         ovs_port_id = ovs_port.ofport
 
         # Delete flow(s) from br-int
-        if direction == 'OUT' or direction == 'BOTH':
+        if direction in ('OUT', 'BOTH'):
             self.int_br.delete_flows(table=0,
                                      in_port=ovs_port_id)
 
-        if direction == 'IN' or direction == 'BOTH':
-            port_mac = tap_flow['port_mac']
+        if direction in ('IN', 'BOTH'):
+            port_mac = tap_flow_msg['port_mac']
 
             #
             # The VLAN id related checks have been temporarily disabled.
@@ -414,8 +457,6 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
             #                                    taas_id,
             #                                    patch_int_tap_id)
 
-        return
-
     def update_tunnel_flood_flow(self):
         flow_action = self._create_tunnel_flood_flow_action()
         if flow_action != "":
@@ -423,11 +464,7 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                                  actions=flow_action)
 
     def _create_tunnel_flood_flow_action(self):
-
-        args = ["ovs-vsctl", "list-ports", "br-tun"]
-        res = utils.execute(args, run_as_root=True)
-
-        port_name_list = res.splitlines()
+        port_name_list = self.tun_br.get_port_name_list()
 
         flow_action = ("move:NXM_OF_VLAN_TCI[0..11]->NXM_NX_TUN_ID[0..11],"
                        "mod_vlan_vid:1")
@@ -435,7 +472,7 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
         tunnel_ports_exist = False
 
         for port_name in port_name_list:
-            if (port_name != 'patch-int') and (port_name != 'patch-tun-tap'):
+            if port_name not in ('patch-int', 'patch-tun-tap'):
                 flow_action += (",output:%d" %
                                 self.tun_br.get_port_ofport(port_name))
                 tunnel_ports_exist = True
@@ -477,8 +514,6 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                              dl_dst="01:00:00:00:00:00/01:00:00:00:00:00",
                              actions=flow_action)
 
-        return
-
     #
     # Removes or updates a special flow in br-int to mirror (duplicate
     # and redirect to 'out_port_id') all ingress broadcast/multicast
@@ -510,5 +545,3 @@ class OvsTaasDriver(taas_base.TaasAgentDriver):
                                      dl_vlan=vlan_id,
                                      dl_dst=("01:00:00:00:00:00/"
                                              "01:00:00:00:00:00"))
-
-        return
